@@ -18,18 +18,15 @@ g++ -DNDEBUG -O3 -Wno-deprecated-declarations $1 $2 $3 \
   src/util/util.cpp  \
   src/util/Timer.cpp \
   src/basic/basic.cpp \
-  src/dp/floating_sw.cpp \
   src/basic/hssp.cpp \
   src/dp/ungapped_align.cpp \
   src/run/tools.cpp \
   src/dp/greedy_align.cpp \
-  src/run/benchmark.cpp \
   src/search/stage2.cpp \
   src/output/output_format.cpp \
   src/output/join_blocks.cpp \
   src/data/frequent_seeds.cpp \
   src/align/query_mapper.cpp \
-  src/align/align_target.cpp \
   src/output/blast_tab_format.cpp \
   src/dp/padded_banded_sw.cpp \
   src/dp/needleman_wunsch.cpp \
@@ -47,10 +44,19 @@ g++ -DNDEBUG -O3 -Wno-deprecated-declarations $1 $2 $3 \
   src/data/taxonomy.cpp \
   src/lib/tantan/tantan.cc \
   src/basic/masking.cpp \
-  src/dp/swipe.cpp \
+  src/dp/swipe/swipe.cpp \
   src/dp/banded_sw.cpp \
   src/data/sorted_list.cpp \
   src/data/seed_set.cpp \
   src/util/binary_file.cpp \
   src/util/simd.cpp \
+  src/output/taxon_format.cpp \
+  src/output/view.cpp \
+  src/output/output_sink.cpp \
+  src/output/target_culling.cpp \
+  src/align/greedy_pipeline.cpp \
+  src/align/swipe_pipeline.cpp \
+  src/dp/swipe/banded_3frame_swipe.cpp \
+  src/dp/swipe/banded_swipe.cpp \
+  src/align/banded_swipe_pipeline.cpp \
 -lz -lpthread -o diamond
