@@ -25,7 +25,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../search/trace_pt_buffer.h"
 #include "../util/task_queue.h"
 #include "../basic/statistics.h"
-#include "align_struct.h"
+#include "query_mapper.h"
 
 using std::vector;
 using std::auto_ptr;
@@ -35,7 +35,7 @@ struct Output_writer
 	Output_writer(Output_stream* f) :
 		f_(f)
 	{ }
-	void operator()(Text_buffer &buf)
+	void operator()(TextBuffer &buf)
 	{
 		f_->write(buf.get_begin(), buf.size());
 		buf.clear();
@@ -64,68 +64,47 @@ private:
 	Task_queue<_buffer, Output_writer> queue;
 };
 
-struct Simple_query_queue
-{
-	enum { end = 0xffffffffffffffffllu };
-	Simple_query_queue(size_t qbegin, size_t qend, vector<hit>::iterator begin, vector<hit>::iterator end):
-		next_(qbegin),
-		qend_(qend),
-		it_(begin),
-		end_(end)
-	{}
-	size_t get(vector<hit>::iterator &begin, vector<hit>::iterator &end);
-	size_t next() const
-	{
-		return next_;
-	}
-	size_t qend() const
-	{
-		return qend_;
-	}
-	static Simple_query_queue& get()
-	{
-		return *instance;
-	}
-	static auto_ptr<Simple_query_queue> instance;
-private:
-	tthread::mutex mtx_;
-	size_t next_;
-	const size_t qend_;
-	vector<hit>::iterator it_, end_;
-};
+void align_queries(Trace_pt_buffer &trace_pts, Output_stream* output_file);
 
-struct Output_sink
-{
-	Output_sink(size_t begin, Output_stream *f):
-		f_(f),
-		next_(begin),
-		size_(0),
-		max_size_(0)
-	{}
-	void push(size_t n, Text_buffer *buf);
-	size_t size() const
-	{
-		return size_;
+namespace ExtensionPipeline {
+	namespace Greedy {
+		struct Target;
+		struct Pipeline : public QueryMapper
+		{
+			Pipeline(size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end) :
+				QueryMapper(query_id, begin, end)
+			{}
+			Target& target(size_t i);
+			virtual void run(Statistics &stat);
+			virtual ~Pipeline() {}
+		};
 	}
-	size_t max_size() const
-	{
-		return max_size_;
+	namespace Swipe {
+		struct Pipeline : public QueryMapper
+		{
+			Pipeline(size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end) :
+				QueryMapper(query_id, begin, end)
+			{}
+			virtual void run(Statistics &stat);
+			virtual ~Pipeline() {}
+		};
 	}
-	static Output_sink& get()
-	{
-		return *instance;
+	namespace BandedSwipe {
+		struct Target;
+		struct Pipeline : public QueryMapper
+		{
+			Pipeline(size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end, DpStat &dp_stat) :
+				QueryMapper(query_id, begin, end),
+				dp_stat(dp_stat)
+			{}
+			Target& target(size_t i);
+			virtual void run(Statistics &stat);
+			void run_swipe(bool score_only);
+			void range_ranking();
+			DpStat &dp_stat;
+		};
 	}
-	size_t next() const
-	{
-		return next_;
-	}
-	static auto_ptr<Output_sink> instance;
-private:
-	void flush(Text_buffer *buf);
-	tthread::mutex mtx_;
-	Output_stream* const f_;
-	std::map<size_t, Text_buffer*> backlog_;
-	size_t next_, size_, max_size_;
-};
+}
+
 
 #endif
\ No newline at end of file
