@@ -16,42 +16,55 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#include "align_queries.h"
 #include "query_mapper.h"
 #include "../data/reference.h"
 #include "extend_ungapped.h"
 #include "../output/output.h"
 #include "../output/output_format.h"
 #include "../output/daa_write.h"
+#include "../output/target_culling.h"
 
-Query_mapper::Query_mapper() :
-	source_hits(get_query_data()),
-	query_id(source_hits.first->query_ / align_mode.query_contexts),
-	targets_finished(0),
-	next_target(0),
-	source_query_len(get_source_query_len(query_id)),
-	unaligned_from(query_queue.last_query+1)
-{	
-	query_queue.last_query = query_id;
-	seed_hits.reserve(source_hits.second - source_hits.first);
+bool Target::envelopes(const Hsp_traits &t, double p) const
+{
+	for (list<Hsp_traits>::const_iterator i = ts.begin(); i != ts.end(); ++i)
+		if (t.query_source_range.overlap_factor(i->query_source_range) >= p)
+			return true;
+	return false;
 }
 
-int Query_mapper::raw_score_cutoff() const
+bool Target::is_enveloped(const Target &t, double p) const
+{
+	for (list<Hsp_traits>::const_iterator i = ts.begin(); i != ts.end(); ++i)
+		if (!t.envelopes(*i, p))
+			return false;
+	return true;
+}
+
+bool Target::is_enveloped(Ptr_vector<Target>::const_iterator begin, Ptr_vector<Target>::const_iterator end, double p, int min_score) const
+{
+	for (; begin != end; ++begin)
+		if (is_enveloped(**begin, p) && (*begin)->filter_score >= min_score)
+			return true;
+	return false;
+}
+
+int QueryMapper::raw_score_cutoff() const
 {
 	return score_matrix.rawscore(config.min_bit_score == 0 ? score_matrix.bitscore(config.max_evalue, ref_header.letters, (unsigned)query_seq(0).length()) : config.min_bit_score);
 }
 
-Query_mapper::Query_mapper(size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end) :
+QueryMapper::QueryMapper(size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end) :
 	source_hits(std::make_pair(begin, end)),
 	query_id((unsigned)query_id),
 	targets_finished(0),
 	next_target(0),
-	source_query_len(get_source_query_len((unsigned)query_id))
+	source_query_len(get_source_query_len((unsigned)query_id)),
+	translated_query(get_translated_query(query_id))
 {
 	seed_hits.reserve(source_hits.second - source_hits.first);
 }
 
-void Query_mapper::init()
+void QueryMapper::init()
 {
 	if(config.log_query)
 		cout << "Query = " << query_ids::get()[query_id].c_str() << endl;
@@ -66,23 +79,9 @@ void Query_mapper::init()
 	if (targets.empty())
 		return;
 	load_targets();
-	if (config.ext == Config::floating_xdrop)
-		rank_targets(config.rank_ratio == -1 ? 0.6 : config.rank_ratio);
 }
 
-pair<Trace_pt_list::iterator, Trace_pt_list::iterator> Query_mapper::get_query_data()
-{
-	const Trace_pt_list::iterator begin = query_queue.trace_pt_pos;
-	if (begin == query_queue.trace_pt_end)
-		return pair<Trace_pt_list::iterator, Trace_pt_list::iterator>(begin, begin);
-	const unsigned c = align_mode.query_contexts, query = begin->query_ / c;
-	Trace_pt_list::iterator end = begin;
-	for (; end < query_queue.trace_pt_end && end->query_ / c == query; ++end);
-	query_queue.trace_pt_pos = end;
-	return pair<Trace_pt_list::iterator, Trace_pt_list::iterator>(begin, end);
-}
-
-unsigned Query_mapper::count_targets()
+unsigned QueryMapper::count_targets()
 {
 	std::sort(source_hits.first, source_hits.second, hit::cmp_subject);
 	const size_t n = source_hits.second - source_hits.first;
@@ -110,14 +109,13 @@ unsigned Query_mapper::count_targets()
 	return n_subject;
 }
 
-void Query_mapper::load_targets()
+void QueryMapper::load_targets()
 {
 	unsigned subject_id = std::numeric_limits<unsigned>::max(), n = 0;
 	for (size_t i = 0; i < seed_hits.size(); ++i) {
 		if (seed_hits[i].subject_ != subject_id) {
 			if (n > 0) {
 				targets[n - 1].end = i;
-				get_prefilter_score(n - 1);
 			}
 			targets.get(n) = new Target(i, seed_hits[i].subject_);
 			++n;
@@ -125,12 +123,11 @@ void Query_mapper::load_targets()
 		}
 	}
 	targets[n - 1].end = seed_hits.size();
-	get_prefilter_score(n - 1);
 }
 
-void Query_mapper::rank_targets(double ratio)
+void QueryMapper::rank_targets(double ratio, double factor)
 {
-	std::sort(targets.begin(), targets.end(), Target::compare);
+	std::stable_sort(targets.begin(), targets.end(), Target::compare);
 
 	int score = 0;
 	if (config.toppercent < 100) {
@@ -141,79 +138,102 @@ void Query_mapper::rank_targets(double ratio)
 		score = int((double)targets[min_idx - 1].filter_score * ratio);
 	}
 
-	unsigned i = 0;
+	const size_t cap = (config.toppercent < 100 || config.max_alignments == std::numeric_limits<uint64_t>::max()) ? std::numeric_limits<uint64_t>::max() : size_t(config.max_alignments*factor);
+	size_t i = 0;
 	for (; i < targets.size(); ++i)
-		if (targets[i].filter_score < score)
+		if (targets[i].filter_score < score || i >= cap)
 			break;
-	
+
 	if (config.benchmark_ranking)
-		for (unsigned j = i; j < targets.size(); ++j)
+		for (size_t j = i; j < targets.size(); ++j)
 			targets[j].outranked = true;
 	else
 		targets.erase(targets.begin() + i, targets.end());
 }
 
-bool Query_mapper::generate_output(Text_buffer &buffer, Statistics &stat)
+void QueryMapper::score_only_culling()
 {
-	if (!blocked_processing && *output_format != Output_format::daa && config.report_unaligned != 0 && config.load_balancing == Config::target_parallel) {
-		for (unsigned i = unaligned_from; i < query_id; ++i) {
-			output_format->print_query_intro(i, query_ids::get()[i].c_str(), get_source_query_len(i), buffer, true);
-			output_format->print_query_epilog(buffer, true);
+	std::stable_sort(targets.begin(), targets.end(), Target::compare);
+	auto_ptr<TargetCulling> target_culling(TargetCulling::get());
+	const unsigned query_len = (unsigned)query_seq(0).length();
+	Ptr_vector<Target>::iterator i;
+	for (i = targets.begin(); i<targets.end();) {
+		if ((config.min_bit_score == 0 && score_matrix.evalue((*i)->filter_score, config.db_size, query_len) > config.max_evalue)
+			|| score_matrix.bitscore((*i)->filter_score) < config.min_bit_score)
+			break;
+		const int c = target_culling->cull(**i);
+		if (c == TargetCulling::FINISHED)
+			break;
+		else if (c == TargetCulling::NEXT) {
+			if (config.benchmark_ranking)
+				(*i++)->outranked = true;
+			else
+				i = targets.erase(i, i + 1);
+		}
+		else {
+			target_culling->add(**i);
+			++i;
 		}
 	}
+	targets.erase(i, targets.end());
+}
 
-	std::sort(targets.begin(), targets.end(), Target::compare);
+bool QueryMapper::generate_output(TextBuffer &buffer, Statistics &stat)
+{
+	std::stable_sort(targets.begin(), targets.end(), Target::compare);
 
 	unsigned n_hsp = 0, n_target_seq = 0, hit_hsps = 0;
-	const unsigned top_score = targets.empty() ? 0 : targets[0].filter_score, query_len = (unsigned)query_seq(0).length();
+	auto_ptr<TargetCulling> target_culling(TargetCulling::get());
+	const unsigned query_len = (unsigned)query_seq(0).length();
 	size_t seek_pos = 0;
 	const char *query_title = query_ids::get()[query_id].c_str();
+	auto_ptr<Output_format> f(output_format->clone());
 
 	for (size_t i = 0; i < targets.size(); ++i) {
 		if ((config.min_bit_score == 0 && score_matrix.evalue(targets[i].filter_score, config.db_size, query_len) > config.max_evalue)
 			|| score_matrix.bitscore(targets[i].filter_score) < config.min_bit_score)
 			break;
 
-		if (!config.output_range(n_target_seq, targets[i].filter_score, top_score))
+		const unsigned subject_len = (unsigned)ref_seqs::get()[targets[i].subject_id].length();
+		const char *ref_title = ref_ids::get()[targets[i].subject_id].c_str();
+		targets[i].apply_filters(source_query_len, subject_len, query_title, ref_title);
+		if (targets[i].hsps.size() == 0)
+			continue;
+
+		const int c = target_culling->cull(targets[i]);
+		if (c == TargetCulling::NEXT)
+			continue;
+		else if (c == TargetCulling::FINISHED)
 			break;
 
 		if (targets[i].outranked)
 			stat.inc(Statistics::OUTRANKED_HITS);
-
-		const unsigned subject_len = (unsigned)ref_seqs::get()[targets[i].subject_id].length();
+		++n_target_seq;
+		target_culling->add(targets[i]);
 		
 		hit_hsps = 0;
-		for (list<Hsp_data>::iterator j = targets[i].hsps.begin(); j != targets[i].hsps.end(); ++j) {
-			if (hit_hsps >= config.max_hsps)
+		for (list<Hsp>::iterator j = targets[i].hsps.begin(); j != targets[i].hsps.end(); ++j) {
+			if (config.max_hsps > 0 && hit_hsps >= config.max_hsps)
 				break;
-			const char *ref_title = ref_ids::get()[targets[i].subject_id].c_str();
-			if (j->id_percent() < config.min_id
-				|| j->query_cover_percent(source_query_len) < config.query_cover
-				|| j->subject_cover_percent(subject_len) < config.subject_cover
-				|| (config.no_self_hits &&
-					(config.ext == Config::more_greedy || (j->identities == j->length && j->query_source_range.length() == (int)source_query_len && j->subject_range.length() == (int)subject_len))
-					&& strcmp(query_title, ref_title) == 0))
-				continue;
 
 			if (blocked_processing) {
 				if (n_hsp == 0)
-					seek_pos = Intermediate_record::write_query_intro(buffer, query_id);
-				Intermediate_record::write(buffer, *j, query_id, targets[i].subject_id);
+					seek_pos = IntermediateRecord::write_query_intro(buffer, query_id);
+				IntermediateRecord::write(buffer, *j, query_id, targets[i].subject_id);
 			}
 			else {
 				if (n_hsp == 0) {
-					if (*output_format == Output_format::daa)
+					if (*f == Output_format::daa)
 						seek_pos = write_daa_query_record(buffer, query_title, align_mode.query_translated ? query_source_seqs::get()[query_id] : query_seqs::get()[query_id]);
 					else
-						output_format->print_query_intro(query_id, query_title, source_query_len, buffer, false);
+						f->print_query_intro(query_id, query_title, source_query_len, buffer, false);
 				}
-				if (*output_format == Output_format::daa)
+				if (*f == Output_format::daa)
 					write_daa_record(buffer, *j, query_id, targets[i].subject_id);
 				else
-					output_format->print_match(Hsp_context(*j,
+					f->print_match(Hsp_context(*j,
 						query_id,
-						query_seq(j->frame),
-						query_source_seq(),
+						translated_query,
 						query_title,
 						targets[i].subject_id,
 						targets[i].subject_id,
@@ -223,29 +243,24 @@ bool Query_mapper::generate_output(Text_buffer &buffer, Statistics &stat)
 						hit_hsps), buffer);
 			}
 
-			if(hit_hsps == 0)
-				++n_target_seq;
 			++n_hsp;
 			++hit_hsps;
-			if (config.alignment_traceback && j->gap_openings > 0)
-				stat.inc(Statistics::GAPPED);
-			stat.inc(Statistics::SCORE_TOTAL, j->score);
 		}
 	}
 
 	if (n_hsp > 0) {
 		if (!blocked_processing) {
-			if (*output_format == Output_format::daa)
+			if (*f == Output_format::daa)
 				finish_daa_query_record(buffer, seek_pos);
 			else
-				output_format->print_query_epilog(buffer, false);
+				f->print_query_epilog(buffer, query_title, false);
 		}
 		else
-			Intermediate_record::finish_query(buffer, seek_pos);
+			IntermediateRecord::finish_query(buffer, seek_pos);
 	}
-	else if (!blocked_processing && *output_format != Output_format::daa && config.report_unaligned != 0) {
-		output_format->print_query_intro(query_id, query_title, source_query_len, buffer, true);
-		output_format->print_query_epilog(buffer, true);
+	else if (!blocked_processing && *f != Output_format::daa && config.report_unaligned != 0) {
+		f->print_query_intro(query_id, query_title, source_query_len, buffer, true);
+		f->print_query_epilog(buffer, query_title, true);
 	}
 
 	if (!blocked_processing) {
@@ -256,4 +271,36 @@ bool Query_mapper::generate_output(Text_buffer &buffer, Statistics &stat)
 	}
 	
 	return n_hsp > 0;
+}
+
+void Target::inner_culling(int cutoff)
+{
+	hsps.sort();
+	if (hsps.size() > 0)
+		filter_score = hsps.front().score;
+	else
+		filter_score = 0;
+	for (list<Hsp>::iterator i = hsps.begin(); i != hsps.end();) {
+		if (i->is_enveloped_by(hsps.begin(), i, 0.5) || (int)i->score < cutoff)
+			i = hsps.erase(i);
+		else
+			++i;
+	}
+}
+
+void Target::apply_filters(int dna_len, int subject_len, const char *query_title, const char *ref_title)
+{
+	for (list<Hsp>::iterator i = hsps.begin(); i != hsps.end();) {
+		if (i->id_percent() < config.min_id
+			|| i->query_cover_percent(dna_len) < config.query_cover
+			|| i->subject_cover_percent(subject_len) < config.subject_cover
+			|| (config.no_self_hits
+				&& i->identities == i->length
+				&& i->query_source_range.length() == (int)dna_len
+				&& i->subject_range.length() == (int)subject_len
+				&& strcmp(query_title, ref_title) == 0))
+			i = hsps.erase(i);
+		else
+			++i;
+	}
 }
\ No newline at end of file
