@@ -27,6 +27,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../data/queries.h"
 #include "../util/ptr_vector.h"
 #include "../dp/dp.h"
+#include "../data/reference.h"
 
 using std::vector;
 using std::pair;
@@ -34,37 +35,48 @@ using std::list;
 
 struct Target
 {
+	Target(int score):
+		filter_score(score)
+	{}
 	Target(size_t begin, unsigned subject_id) :
 		subject_id(subject_id),
+		subject(ref_seqs::get()[subject_id]),
 		filter_score(0),
 		outranked(false),
 		begin(begin)
 	{}		
 	static bool compare(Target* lhs, Target *rhs)
 	{
-		return lhs->filter_score > rhs->filter_score;
+		return lhs->filter_score > rhs->filter_score || (lhs->filter_score == rhs->filter_score && lhs->subject_id < rhs->subject_id);
 	}
+	void fill_source_ranges(size_t query_len)
+	{
+		for (list<Hsp_traits>::iterator i = ts.begin(); i != ts.end(); ++i)
+			i->query_source_range = TranslatedPosition::absolute_interval(TranslatedPosition(i->query_range.begin_, Frame(i->frame)), TranslatedPosition(i->query_range.end_, Frame(i->frame)), (int)query_len);
+	}
+	bool envelopes(const Hsp_traits &t, double p) const;
+	bool is_enveloped(const Target &t, double p) const;
+	bool is_enveloped(Ptr_vector<Target>::const_iterator begin, Ptr_vector<Target>::const_iterator end, double p, int min_score) const;
+	void inner_culling(int cutoff);
+	void apply_filters(int dna_len, int subject_len, const char *query_title, const char *ref_title);
 	unsigned subject_id;
+	sequence subject;
 	int filter_score;
 	float filter_time;
 	bool outranked;
 	size_t begin, end;
-	list<Hsp_data> hsps;
+	list<Hsp> hsps;
 	list<Hsp_traits> ts;
+	Seed_hit top_hit;
 };
 
-struct Query_mapper
+struct QueryMapper
 {
-	Query_mapper();
-	Query_mapper(size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end);
+	QueryMapper(size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end);
 	void init();
-	void get_prefilter_score(size_t idx);
-	void align_target(size_t idx, Statistics &stat);
-	void align_targets(Statistics &stat);
-	bool generate_output(Text_buffer &buffer, Statistics &stat);
-	void ungapped_stage(size_t idx);
-	void greedy_stage(size_t idx, Statistics &stat, int cutoff);
-	void rank_targets(double ratio);
+	bool generate_output(TextBuffer &buffer, Statistics &stat);
+	void rank_targets(double ratio, double factor);
+	void score_only_culling();
 	int raw_score_cutoff() const;
 	size_t n_targets() const
 	{
@@ -78,9 +90,24 @@ struct Query_mapper
 	{
 		return query_seqs::get()[query_id*align_mode.query_contexts + frame];
 	}
+	void fill_source_ranges()
+	{
+		for (size_t i = 0; i < targets.size(); ++i)
+			targets[i].fill_source_ranges(source_query_len);
+	}
+	virtual void run(Statistics &stat) = 0;
+	virtual ~QueryMapper() {}
 	pair<Trace_pt_list::iterator, Trace_pt_list::iterator> source_hits;
 	unsigned query_id, targets_finished, next_target;
+	unsigned source_query_len, unaligned_from;
+	Ptr_vector<Target> targets;
+	vector<Seed_hit> seed_hits;
+	vector<Bias_correction> query_cb;
+	vector<Long_score_profile> profile;
+	TranslatedSequence translated_query;
+
 private:
+
 	static pair<Trace_pt_list::iterator, Trace_pt_list::iterator> get_query_data();
 	unsigned count_targets();
 	sequence query_source_seq() const
@@ -88,12 +115,6 @@ private:
 		return align_mode.query_translated ? query_source_seqs::get()[query_id] : query_seqs::get()[query_id];
 	}
 	void load_targets();
-
-	unsigned source_query_len, unaligned_from;
-	vector<Seed_hit> seed_hits;
-	Ptr_vector<Target> targets;
-	vector<Bias_correction> query_cb;
-	vector<Long_score_profile> profile;
 	
 };
 
