@@ -56,7 +56,9 @@ Config::Config(int argc, const char **argv)
 		.add_command("model-seqs", "")
 		.add_command("opt", "")
 		.add_command("mask", "")
-		.add_command("fastq2fasta", "");
+		.add_command("fastq2fasta", "")
+		.add_command("dbinfo", "Print information about a DIAMOND database file")
+		.add_command("test-extra", "");
 
 	Options_group general("General options");
 	general.add()
@@ -111,10 +113,12 @@ Config::Config(int argc, const char **argv)
 	Options_group aligner("Aligner options");
 	aligner.add()
 		("query", 'q', "input query file", query_file)
+		("strand", 0, "query strands to search (both/minus/plus)", query_strands, string("both"))
 		("un", 0, "file for unaligned queries", unaligned)
 		("unal", 0, "report unaligned queries (0=no, 1=yes)", report_unaligned, -1)
 		("max-target-seqs", 'k', "maximum number of target sequences to report alignments for", max_alignments, uint64_t(25))
 		("top", 0, "report alignments within this percentage range of top alignment score (overrides --max-target-seqs)", toppercent, 100.0)
+		("range-culling", 0, "restrict hit culling to overlapping query ranges", query_range_culling)
 		("compress", 0, "compression for output files (0=none, 1=gzip)", compression)
 		("evalue", 'e', "maximum e-value to report alignments (default=0.001)", max_evalue, 0.001)
 		("min-score", 0, "minimum bit score to report alignments (overrides e-value setting)", min_bit_score)
@@ -128,6 +132,7 @@ Config::Config(int argc, const char **argv)
 		("tmpdir", 't', "directory for temporary files", tmpdir)
 		("gapopen", 0, "gap open penalty", gap_open, -1)
 		("gapextend", 0, "gap extension penalty", gap_extend, -1)
+		("frameshift", 'F', "frame shift penalty (default=disabled)", frame_shift)
 		("matrix", 0, "score matrix for protein alignment (default=BLOSUM62)", matrix, string("blosum62"))
 		("custom-matrix", 0, "file containing custom scoring matrix", matrix_file)
 		("lambda", 0, "lambda parameter for custom matrix", lambda)
@@ -137,8 +142,10 @@ Config::Config(int argc, const char **argv)
 		//("seg", 0, "enable SEG masking of queries (yes/no)", seg)
 		("query-gencode", 0, "genetic code to use to translate query (see user manual)", query_gencode, 1u)
 		("salltitles", 0, "include full subject titles in DAA file", salltitles)
+		("sallseqid", 0, "include all subject ids in DAA file", sallseqid)
 		("no-self-hits", 0, "suppress reporting of identical self hits", no_self_hits)
-		("taxonmap", 0, "protein accession to taxid mapping file", prot_accession2taxid);
+		("taxonmap", 0, "protein accession to taxid mapping file", prot_accession2taxid)
+		("taxonnodes", 0, "taxonomy nodes.dmp from NCBI", nodesdmp);
 
 	Options_group advanced("Advanced options");
 	advanced.add()
@@ -159,9 +166,11 @@ Config::Config(int argc, const char **argv)
 		("index-mode", 0, "index mode (0=4x12, 1=16x9)", index_mode)
 		("rank-ratio", 0, "include subjects within this ratio of last hit (stage 1)", rank_ratio, -1.0)
 		("rank-ratio2", 0, "include subjects within this ratio of last hit (stage 2)", rank_ratio2, -1.0)
-		("max-hsps", 0, "maximum number of HSPs per subject sequence to save for each query", max_hsps, 1u)
+		("max-hsps", 0, "maximum number of HSPs per subject sequence to save for each query", max_hsps, 0u)
+		("range-cover", 0, "percentage of query range to be covered for hit culling (default=50)", query_range_cover, 50.0)
 		("dbsize", 0, "effective database size (in letters)", db_size)
-		("no-auto-append", 0, "disable auto appending of DAA and DMND file extensions", no_auto_append);
+		("no-auto-append", 0, "disable auto appending of DAA and DMND file extensions", no_auto_append)
+		("xml-blord-format", 0, "Use gnl|BL_ORD_ID| style format in XML output", xml_blord_format);
 
 	Options_group view_options("View options");
 	view_options.add()
@@ -217,7 +226,10 @@ Config::Config(int argc, const char **argv)
 		("fetch-size", 0, "trace point fetch size", fetch_size, 4096u)
 		("target-fetch-size", 0, "number of target sequences to fetch for seed extension", target_fetch_size, 4u)
 		("small-query", 0, "", small_query)
-		("hashed-seeds", 0, "", hashed_seeds);
+		("hashed-seeds", 0, "", hashed_seeds)
+		("rank-factor", 0, "", rank_factor, -1.0)
+		("transcript-len-estimate", 0, "", transcript_len_estimate, 1.0)
+		("no-traceback", 0, "", disable_traceback);
 		
 	parser.add(general).add(makedb).add(aligner).add(advanced).add(view_options).add(getseq_options).add(hidden_options);
 	parser.store(argc, argv, command);
@@ -230,7 +242,7 @@ Config::Config(int argc, const char **argv)
 			throw std::runtime_error("Invalid option: --block-size/-b. Block size is set for the alignment commands.");
 		break;
 	case Config::blastp:
-	case Config::blastx:	
+	case Config::blastx:
 		if (database == "")
 			throw std::runtime_error("Missing parameter: database file (--db/-d)");
 		if (daa_file.length() > 0) {
@@ -254,6 +266,12 @@ Config::Config(int argc, const char **argv)
 		;
 	}
 
+	switch (command) {
+	case Config::dbinfo:
+		if (database == "")
+			throw std::runtime_error("Missing parameter: database file (--db/-d)");
+	}
+
 	if (hit_cap != 0)
 		throw std::runtime_error("Deprecated parameter: --max-hits/-C.");
 
@@ -322,8 +340,12 @@ Config::Config(int argc, const char **argv)
 	case Config::opt:
 	case Config::mask:
 	case Config::makedb:
+		if (frame_shift != 0 && command == Config::blastp)
+			throw std::runtime_error("Frameshift alignments are only supported for translated searches.");
+		if (query_range_culling && frame_shift == 0)
+			throw std::runtime_error("Query range culling is only supported in frameshift alignment mode (option -F).");
 		if (matrix_file == "")
-			score_matrix = Score_matrix(to_upper_case(matrix), gap_open, gap_extend, reward, penalty);
+			score_matrix = Score_matrix(to_upper_case(matrix), gap_open, gap_extend, reward, penalty, frame_shift);
 		else {
 			if (lambda == 0 || K == 0)
 				throw std::runtime_error("Custom scoring matrices require setting the --lambda and --K options.");
@@ -343,24 +365,7 @@ Config::Config(int argc, const char **argv)
 		
 		init_cbs();
 		raw_ungapped_xdrop = score_matrix.rawscore(ungapped_xdrop);
-
-#ifdef __SSSE3__
-		verbose_stream << "SSSE3 enabled." << endl;
-#endif
-#ifdef __SSE4_1__
-		verbose_stream << "SSE4.1 enabled." << endl;
-#endif
-#ifdef __POPCNT__
-		verbose_stream << "POPCNT enabled." << endl;
-#endif
-
-		message_stream << "#Target sequences to report alignments for: ";
-		if (max_alignments == 0) {
-			max_alignments = std::numeric_limits<uint64_t>::max();
-			message_stream << "unlimited" << endl;
-		}
-		else
-			message_stream << max_alignments << endl;
+		simd_messages();
 	}
 
 	Translator::init(query_gencode);
@@ -371,6 +376,9 @@ Config::Config(int argc, const char **argv)
 	if (command == help)
 		parser.print_help();
 
+	if (query_strands != "both" && query_strands != "minus" && query_strands != "plus")
+		throw std::runtime_error("Invalid value for parameter --strand");
+
 	/*log_stream << "sizeof(hit)=" << sizeof(hit) << " sizeof(packed_uint40_t)=" << sizeof(packed_uint40_t)
 		<< " sizeof(sorted_list::entry)=" << sizeof(sorted_list::entry) << endl;*/
 }
