@@ -59,7 +59,7 @@ struct Config
 	unsigned	seed_signatures;
 	double	min_bit_score;
 	unsigned	run_len;
-	bool		alignment_traceback;
+	bool		disable_traceback;
 	double	max_seed_freq;
 	string	tmpdir;
 	bool		long_mode;
@@ -94,6 +94,7 @@ struct Config
 	vector<string> seq_no;
 	double rank_ratio;
 	double rank_ratio2;
+	double rank_factor;
 	bool ht_mode;
 	bool old_freq;
 	double freq_sd;
@@ -139,10 +140,18 @@ struct Config
 	double score_ratio;
 	bool small_query;
 	bool hashed_seeds;
+	string nodesdmp;
+	bool sallseqid;
+	string query_strands;
+	bool xml_blord_format;
+	int frame_shift;
+	bool query_range_culling;
+	double query_range_cover;
+	double transcript_len_estimate;
 
 	enum {
 		makedb = 0, blastp = 1, blastx = 2, view = 3, help = 4, version = 5, getseq = 6, benchmark = 7, random_seqs = 8, compare = 9, sort = 10, roc = 11, db_stat = 12, model_sim = 13,
-		match_file_stat = 14, model_seqs = 15, opt = 16, mask = 17, fastq2fasta=18
+		match_file_stat = 14, model_seqs = 15, opt = 16, mask = 17, fastq2fasta=18, dbinfo=19, test_extra = 20
 	};
 	unsigned	command;
 
@@ -153,7 +162,7 @@ struct Config
 	unsigned load_balancing;
 
 	enum {
-		swipe = 0, greedy = 1, floating_xdrop = 4, more_greedy = 2, most_greedy=3,
+		swipe = 0, greedy = 1, floating_xdrop = 4, more_greedy = 2, most_greedy=3, banded_swipe=4
 	};
 	int ext;
 
