@@ -20,6 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define MATCH_H_
 
 #include <limits>
+#include <list>
 #include "sequence.h"
 #include "../util/util.h"
 #include "../util/async_buffer.h"
@@ -28,8 +29,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "value.h"
 #include "packed_transcript.h"
 #include "score_matrix.h"
-
-enum Strand { FORWARD, REVERSE };
+#include "translated_position.h"
+#include "diagonal_segment.h"
 
 inline interval normalized_range(unsigned pos, int len, Strand strand)
 {
@@ -38,110 +39,12 @@ inline interval normalized_range(unsigned pos, int len, Strand strand)
 			: interval (pos + 1 + len, pos + 1);
 }
 
-struct Diagonal_segment
-{
-	Diagonal_segment():
-		len(0)
-	{}
-	Diagonal_segment(int query_pos, int subject_pos, int len, int score):
-		i(query_pos),
-		j(subject_pos),
-		len(len),
-		score (score)
-	{}
-	bool empty() const
-	{
-		return len == 0;
-	}
-	interval query_range() const
-	{
-		return interval(i, i + len);
-	}
-	interval subject_range() const
-	{
-		return interval(j, j + len);
-	}
-	int subject_last() const
-	{
-		return j + len - 1;
-	}
-	int query_last() const
-	{
-		return i + len - 1;
-	}
-	int subject_end() const
-	{
-		return j + len;
-	}
-	int query_end() const
-	{
-		return i + len;
-	}
-	int diag() const
-	{
-		return i - j;
-	}
-	Diagonal_segment intersect(const Diagonal_segment &x) const
-	{
-		if (diag() != x.diag())
-			return Diagonal_segment();
-		else {
-			const interval q = ::intersect(query_range(), x.query_range());
-			return Diagonal_segment(q.begin_, ::intersect(subject_range(), x.subject_range()).begin_, q.length(), 0);
-		}
-	}
-	bool is_enveloped(const Diagonal_segment &x) const
-	{
-		return score <= x.score
-			&& query_range().overlap_factor(x.query_range()) == 1
-			&& subject_range().overlap_factor(x.subject_range()) == 1;
-	}
-	Diagonal_segment transpose() const
-	{
-		return Diagonal_segment(j, i, len, score);
-	}
-	int partial_score(int diff) const
-	{
-		return score*std::max(len - diff, 0) / len;
-	}
-	bool operator<=(const Diagonal_segment &rhs) const
-	{
-		return i + len <= rhs.i && j + len <= rhs.j;
-	}
-	bool operator==(const Diagonal_segment &rhs) const
-	{
-		return i == rhs.i && j == rhs.j && len == rhs.len;
-	}
-	static bool cmp_subject(const Diagonal_segment &x, const Diagonal_segment &y)
-	{
-		return x.j < y.j || (x.j == y.j && x.i < y.i);
-	}
-	static bool cmp_subject_end(const Diagonal_segment &x, const Diagonal_segment &y)
-	{
-		return x.subject_end() < y.subject_end();
-	}
-	static bool cmp_heuristic(const Diagonal_segment &x, const Diagonal_segment &y)
-	{
-		return (x.subject_end() < y.subject_end() && x.j < y.j)
-			|| (x.j - y.j < y.subject_end() - x.subject_end());
-	}
-	friend int abs_shift(const Diagonal_segment &x, const Diagonal_segment &y)
-	{
-		return abs(x.diag() - y.diag());
-	}
-	friend std::ostream& operator<<(std::ostream &s, const Diagonal_segment &d)
-	{
-		s << "i=" << d.i << " j=" << d.j << " l=" << d.len << " score=" << d.score;
-		return s;
-	}
-	int i, j, len, score;
-};
+struct IntermediateRecord;
 
-struct Intermediate_record;
-
-struct Hsp_data
+struct Hsp
 {
-	Hsp_data() :
+
+	Hsp() :
 		score(0),
 		frame(0),
 		length(0),
@@ -152,7 +55,8 @@ struct Hsp_data
 		gaps(0),
 		sw_score(0)
 	{}
-	Hsp_data(int score) :
+
+	Hsp(int score) :
 		score(unsigned(score)),
 		frame(0),
 		length(0),
@@ -163,11 +67,13 @@ struct Hsp_data
 		gaps(0),
 		sw_score(0)
 	{}
-	Hsp_data(const Intermediate_record &r, unsigned query_source_len);
+
+	Hsp(const IntermediateRecord &r, unsigned query_source_len);
+
 	struct Iterator
 	{
-		Iterator(const Hsp_data &parent):
-			query_pos(parent.query_range.begin_),
+		Iterator(const Hsp &parent) :
+			query_pos(parent.query_range.begin_, Frame(parent.frame)),
 			subject_pos(parent.subject_range.begin_),
 			ptr_(parent.transcript.ptr()),
 			count_(ptr_->count())
@@ -189,6 +95,13 @@ struct Hsp_data
 			case op_substitution:
 				++query_pos;
 				++subject_pos;
+				break;
+			case op_frameshift_forward:
+				query_pos.shift_forward();
+				break;
+			case op_frameshift_reverse:
+				query_pos.shift_back();
+				break;
 			}
 			--count_;
 			if (count_ == 0) {
@@ -208,20 +121,18 @@ struct Hsp_data
 			} while (good() && (op() == op_deletion || op() == op_insertion));
 			return good();
 		}
-		unsigned query_pos, subject_pos;
+		TranslatedPosition query_pos;
+		unsigned subject_pos;
 	protected:
 		const Packed_operation *ptr_;
 		unsigned count_;
 	};
+
 	Iterator begin() const
 	{
 		return Iterator(*this);
 	}
-	void set_source_range(unsigned frame, unsigned dna_len);
-	void set_source_range(unsigned oriented_query_begin)
-	{
-		query_source_range = frame < 3 ? interval(oriented_query_begin, oriented_query_begin + 3 * query_range.length()) : interval(oriented_query_begin + 1 - 3 * query_range.length(), oriented_query_begin + 1);
-	}
+
 	interval oriented_range() const
 	{
 		if (frame < 3)
@@ -229,6 +140,7 @@ struct Hsp_data
 		else
 			return interval(query_source_range.end_ - 1, query_source_range.begin_);
 	}
+
 	void set_translated_query_begin(unsigned oriented_query_begin, unsigned dna_len)
 	{
 		int f = frame <= 2 ? frame + 1 : 2 - frame;
@@ -237,34 +149,62 @@ struct Hsp_data
 		else
 			query_range.begin_ = (dna_len + f - oriented_query_begin) / 3;
 	}
+
 	int blast_query_frame() const
 	{
 		return align_mode.query_translated ? (frame <= 2 ? (int)frame + 1 : 2 - (int)frame) : 0;
 	}
-	bool operator<(const Hsp_data &rhs) const
+
+	bool operator<(const Hsp &rhs) const
 	{
 		return score > rhs.score;
 	}
+
 	double id_percent() const
 	{
 		return (double)identities * 100.0 / (double)length;
 	}
+
 	double query_cover_percent(unsigned query_source_len) const
 	{
 		return (double)query_source_range.length() * 100 / query_source_len;
 	}
+
 	double subject_cover_percent(unsigned subject_len) const
 	{
 		return (double)subject_range.length() * 100 / subject_len;
 	}
-	static bool cmp_query_pos(const Hsp_data &x, const Hsp_data &y)
+
+	static bool cmp_query_pos(const Hsp &x, const Hsp &y)
 	{
 		return x.query_range.begin_ < y.query_range.begin_;
 	}
-	bool pass_through(const Diagonal_segment &d) const;
-	bool is_weakly_enveloped(const Hsp_data &j) const;
+
+	int partial_score(const Hsp &h) const
+	{
+		const double overlap = std::max(subject_range.overlap_factor(h.subject_range), query_source_range.overlap_factor(h.query_source_range));
+		return int((1 - overlap)*score);
+	}
+
+	bool envelopes(const DiagonalSegment &d, int dna_len) const
+	{
+		return query_source_range.contains(d.query_absolute_range(dna_len)) || subject_range.contains(d.subject_range());
+	}
+
+	bool is_enveloped_by(const Hsp &hsp, double p) const;
+	bool is_enveloped_by(std::list<Hsp>::const_iterator begin, std::list<Hsp>::const_iterator end, double p) const;
+	bool is_weakly_enveloped_by(std::list<Hsp>::const_iterator begin, std::list<Hsp>::const_iterator end, int cutoff) const;
+	void push_back(const DiagonalSegment &d, const TranslatedSequence &query, const sequence &subject, bool reversed);
+	void push_match(Letter q, Letter s, bool positive);
+	void splice(const DiagonalSegment &d0, const DiagonalSegment &d1, const TranslatedSequence &query, const sequence &subject, bool reversed);
+	void set_begin(const DiagonalSegment &d, int dna_len);
+	void set_end(const DiagonalSegment &d, int dna_len);
+	void set_begin(int i, int j, Frame frame, int dna_len);
+	void set_end(int i, int j, Frame frame, int dna_len);
+	void clear();
+
+	bool is_weakly_enveloped(const Hsp &j) const;
 	std::pair<int, int> diagonal_bounds() const;
-	void merge(const Hsp_data &right, const Hsp_data &left, unsigned query_anchor, unsigned subject_anchor);
 	unsigned score, frame, length, identities, mismatches, positives, gap_openings, gaps, sw_score;
 	float time;
 	interval query_source_range, query_range, subject_range;
@@ -273,9 +213,19 @@ struct Hsp_data
 
 struct Hsp_context
 {
-	Hsp_context(Hsp_data& hsp, unsigned query_id, const sequence &query, const sequence &source_query, const char *query_name, unsigned subject_id, unsigned orig_subject_id, const char *subject_name, unsigned subject_len, unsigned hit_num, unsigned hsp_num) :		
+	Hsp_context(
+		Hsp& hsp,
+		unsigned query_id,
+		const TranslatedSequence &query,
+		const char *query_name,
+		unsigned subject_id,
+		unsigned orig_subject_id,
+		const char *subject_name,
+		unsigned subject_len,
+		unsigned hit_num,
+		unsigned hsp_num
+	) :
 		query(query),
-		source_query(source_query),
 		query_name(query_name),
 		subject_name(subject_name),
 		query_id(query_id),
@@ -286,10 +236,10 @@ struct Hsp_context
 		hsp_num(hsp_num),
 		hsp_(hsp)
 	{}
-	struct Iterator : public Hsp_data::Iterator
+	struct Iterator : public Hsp::Iterator
 	{
 		Iterator(const Hsp_context &parent) :
-			Hsp_data::Iterator(parent.hsp_),
+			Hsp::Iterator(parent.hsp_),
 			parent_(parent)
 		{ }
 		Letter query() const
@@ -311,6 +261,10 @@ struct Hsp_context
 			switch (op()) {
 			case op_deletion:
 				return '-';
+			case op_frameshift_forward:
+				return '\\';
+			case op_frameshift_reverse:
+				return '/';
 			default:
 				return value_traits.alphabet[(long)query()];
 			}
@@ -319,6 +273,8 @@ struct Hsp_context
 		{
 			switch (op()) {
 			case op_insertion:
+			case op_frameshift_forward:
+			case op_frameshift_reverse:
 				return '-';
 			default:
 				return value_traits.alphabet[(long)subject()];
@@ -354,7 +310,7 @@ struct Hsp_context
 	{ return hsp_.score; }
 	double evalue() const
 	{
-		return score_matrix.evalue(score(), config.db_size, (unsigned)query.length());
+		return score_matrix.evalue(score(), config.db_size, (unsigned)query.index(0).length());
 	}
 	double bit_score() const
 	{
@@ -395,13 +351,12 @@ struct Hsp_context
 	Packed_transcript transcript() const
 	{ return hsp_.transcript; }
 	Hsp_context& parse();
-	Hsp_context& set_query_source_range(unsigned oriented_query_begin);
 
-	const sequence query, source_query;
+	const TranslatedSequence query;
 	const char *query_name, *subject_name;
 	const unsigned query_id, subject_id, orig_subject_id, subject_len, hit_num, hsp_num;
 private:	
-	Hsp_data &hsp_;
+	Hsp &hsp_;
 };
 
 #endif /* MATCH_H_ */
