@@ -199,15 +199,17 @@ const Matrix_info Matrix_info::matrices[] = {
 	{ "PAM30", pam30_values, (const char*)NCBISM_Pam30.scores, PAM30_VALUES_MAX, 9, 1 }
 };
 
-Score_matrix::Score_matrix(const string & matrix, int gap_open, int gap_extend, int reward, int penalty):
+Score_matrix::Score_matrix(const string & matrix, int gap_open, int gap_extend, int reward, int penalty, int frameshift):
 	gap_open_ (gap_open == -1 ? Matrix_info::get(matrix).default_gap_open : gap_open),
 	gap_extend_ (gap_extend == -1 ? Matrix_info::get(matrix).default_gap_extend : gap_extend),
+	frame_shift_(frameshift),
 	constants_ (Matrix_info::get(matrix).get_constants(gap_open_, gap_extend_)),
 	name_(matrix),
 	matrix8_(Matrix_info::get(matrix).scores),
 	bias_((char)(-low_score())),
 	matrix8u_(Matrix_info::get(matrix).scores, bias_),
-	matrix16_(Matrix_info::get(matrix).scores)
+	matrix16_(Matrix_info::get(matrix).scores),
+	matrix32_(Matrix_info::get(matrix).scores)
 { }
 
 char Score_matrix::low_score() const
@@ -219,6 +221,15 @@ char Score_matrix::low_score() const
 	return low;
 }
 
+char Score_matrix::high_score() const
+{
+	char high = std::numeric_limits<char>::min();
+	for (Letter i = 0; i < (char)value_traits.alphabet_size; ++i)
+		for (Letter j = i; j < (char)value_traits.alphabet_size; ++j)
+			high = std::max(high, (char)this->operator()(i, j));
+	return high;
+}
+
 double Score_matrix::avg_id_score() const
 {
 	double s = 0;
@@ -284,7 +295,8 @@ Score_matrix::Score_matrix(const string &matrix_file, double lambda, double K, i
 	matrix8_(custom_scores(matrix_file)),
 	bias_((char)(-low_score())),
 	matrix8u_(custom_scores(matrix_file), bias_),
-	matrix16_(custom_scores(matrix_file))
+	matrix16_(custom_scores(matrix_file)),
+	matrix32_(custom_scores(matrix_file))
 {
 	static double constants[5];
 	constants[3] = lambda;
