@@ -36,7 +36,7 @@ struct Score_matrix
 {
 
 	Score_matrix() {}
-	Score_matrix(const string &matrix, int gap_open, int gap_extend, int reward, int penalty);
+	Score_matrix(const string &matrix, int gap_open, int gap_extend, int reward, int penalty, int frame_shift);
 	Score_matrix(const string &matrix_file, double lambda, double K, int gap_open, int gap_extend);
 
 	friend std::ostream& operator<<(std::ostream& s, const Score_matrix &m);
@@ -50,11 +50,21 @@ struct Score_matrix
 	const int16_t* matrix16() const
 	{ return matrix16_.data; }
 
+	const int* matrix32() const
+	{
+		return matrix32_.data;
+	}
+
 	int operator()(Letter a, Letter b) const
 	{
 		return matrix8_.data[(int(a) << 5) + int(b)];
 	}
 
+	const int* row(char a) const
+	{
+		return &matrix32_.data[(int)a << 5];
+	}
+
 	uint8_t biased_score(Letter a, Letter b) const
 	{ return matrix8u_.data[(int(a) << 5) + int(b)]; }
 
@@ -92,6 +102,7 @@ struct Score_matrix
 	}
 
 	char low_score() const;
+	char high_score() const;
 
 	int gap_open() const
 	{
@@ -103,6 +114,11 @@ struct Score_matrix
 		return gap_extend_;
 	}
 
+	int frame_shift() const
+	{
+		return frame_shift_;
+	}
+
 	double avg_id_score() const;
 
 private:
@@ -114,9 +130,9 @@ private:
 		Scores(const char *scores, char bias = 0)
 		{
 			const unsigned n = value_traits.alphabet_size;
-			for(unsigned i=0;i<32;++i)
-				for(unsigned j=0;j<32;++j)
-					data[i*32+j] = i < n && j < n ? (_t)(scores[i*n+j] + (int)bias) : std::numeric_limits<_t>::min();
+			for (unsigned i = 0; i < 32; ++i)
+				for (unsigned j = 0; j < 32; ++j)
+					data[i * 32 + j] = i < n && j < n ? (_t)(scores[i*n + j] + (int)bias) : -(std::numeric_limits<_t>::max() / 2);
 		}
 #ifdef _MSC_VER
 		__declspec(align(16)) _t data[32 * 32];
@@ -125,13 +141,14 @@ private:
 #endif
 	};
 
-	int gap_open_, gap_extend_;
+	int gap_open_, gap_extend_, frame_shift_;
 	const double *constants_;
 	string name_;
 	Scores<int8_t> matrix8_;
 	char bias_;
 	Scores<uint8_t> matrix8u_;
 	Scores<int16_t> matrix16_;
+	Scores<int> matrix32_;
 
 };
 
