@@ -24,11 +24,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../basic/value.h"
 #include "../util/binary_buffer.h"
 #include "../util/text_buffer.h"
+#include "translated_position.h"
 
 using std::vector;
 
 struct sequence
 {
+	static const char DELIMITER = '\x1f';
 	struct Reversed {};
 	sequence():
 		len_ (0),
@@ -92,7 +94,7 @@ struct sequence
 			*(ptr++) = to_char(data_[i]);
 		return len;
 	}
-	Text_buffer& print(Text_buffer &buf, size_t begin, size_t end, const Value_traits& value_traits) const
+	TextBuffer& print(TextBuffer &buf, size_t begin, size_t end, const Value_traits& value_traits) const
 	{
 		for (size_t i = begin; i < end; ++i)
 			buf << value_traits.alphabet[(long)data_[i]];
@@ -100,8 +102,13 @@ struct sequence
 	}
 	std::ostream& print(std::ostream &os, const Value_traits &v) const
 	{
-		for (unsigned i = 0; i<len_; ++i)
-			os << v.alphabet[(long)data_[i]];
+		for (unsigned i = 0; i < len_; ++i) {
+			long l = (long)data_[i];
+			if ((l & 128) == 0)
+				os << v.alphabet[l];
+			else
+				os << (char)tolower(v.alphabet[l & 127]);
+		}
 		return os;
 	}
 	std::ostream& print(std::ostream &os, const Value_traits &v, Reversed) const
@@ -118,7 +125,7 @@ struct sequence
 	{
 		return s.print(os, value_traits);
 	}
-	friend Text_buffer& operator<<(Text_buffer &buf, const sequence &s)
+	friend TextBuffer& operator<<(TextBuffer &buf, const sequence &s)
 	{
 		for(unsigned i=0;i<s.len_;++i)
 			buf << value_traits.alphabet[(long)s.data_[i]];
@@ -128,12 +135,12 @@ struct sequence
 	{
 		const Letter *p = s;
 		int n = 0;
-		while (*p != '\xff' && n < window) {
+		while (*p != sequence::DELIMITER && n < window) {
 			--p;
 			++n;
 		}
 		n = 0;
-		while (*s != '\xff' && n < window) {
+		while (*s != sequence::DELIMITER && n < window) {
 			++s;
 			++n;
 		}
@@ -149,11 +156,87 @@ struct sequence
 		}
 		return os;
 	}*/
-	static vector<Letter> from_string(const char* str);
+	static vector<Letter> from_string(const char* str, const Value_traits &vt = value_traits);
 	size_t			len_;
 	int				clipping_offset_;
 	const Letter	*data_;
 };
 
+struct TranslatedSequence
+{
+
+	TranslatedSequence()
+	{}
+
+	explicit TranslatedSequence(const sequence &s1):
+		source_(s1)
+	{
+		translated_[0] = s1;
+	}
+
+	TranslatedSequence(const sequence &source, const sequence &s1, const sequence &s2, const sequence &s3, const sequence &s4, const sequence &s5, const sequence &s6):
+		source_(source)
+	{
+		translated_[0] = s1;
+		translated_[1] = s2;
+		translated_[2] = s3;
+		translated_[3] = s4;
+		translated_[4] = s5;
+		translated_[5] = s6;
+	}
+
+	TranslatedSequence(const sequence &source, const vector<Letter> v[6]):
+		source_(source)
+	{
+		for (int i = 0; i < 6; ++i)
+			translated_[i] = sequence(v[i]);
+	}
+
+	const sequence& operator[](Frame frame) const
+	{
+		return translated_[frame.index()];
+	}
+
+	const Letter& operator[](const TranslatedPosition &i) const
+	{
+		return (*this)[i.frame][i];
+	}
+
+	const Letter& operator()(int in_strand, Strand strand) const
+	{
+		assert(in_strand < (int)source_.length() - 2);
+		return translated_[in_strand % 3 + (strand == FORWARD ? 0 : 3)][in_strand / 3];
+	}
+
+	const sequence& index(unsigned frame) const
+	{
+		return translated_[frame];
+	}
+
+	const sequence &source() const
+	{
+		return source_;
+	}
+
+	bool in_bounds(const TranslatedPosition &i) const
+	{
+		return i >= 0 && i < int((*this)[i.frame].length());
+	}
+
+	void get_strand(Strand strand, sequence *dst) const
+	{
+		int i = strand == FORWARD ? 0 : 3;
+		dst[0] = translated_[i++];
+		dst[1] = translated_[i++];
+		dst[2] = translated_[i];
+	}
+
+private:
+	
+	sequence source_;
+	sequence translated_[6];
+
+};
+
 
 #endif /* SEQUENCE_H_ */
