@@ -39,6 +39,18 @@ bool blocked_processing;
 using std::cout;
 using std::endl;
 
+string* get_allseqids(const char *s)
+{
+	string *r = new string;
+	const vector<string> t(tokenize(s, "\1"));
+	for (vector<string>::const_iterator i = t.begin(); i != t.end(); ++i) {
+		if (i != t.begin())
+			r->append("\1");
+		r->append(i->substr(0, find_first_of(i->c_str(), Const::id_delimiters)));
+	}
+	return r;
+}
+
 struct Pos_record
 {
 	Pos_record()
@@ -167,6 +179,8 @@ bool Database_file::load_seqs()
 
 	for (size_t n = 0; n < seqs; ++n) {
 		read(ref_seqs::data_->ptr(n) - 1, ref_seqs::data_->length(n) + 2);
+		*(ref_seqs::data_->ptr(n) - 1) = sequence::DELIMITER;
+		*(ref_seqs::data_->ptr(n) + ref_seqs::data_->length(n)) = sequence::DELIMITER;
 		read(ref_ids::data_->ptr(n), ref_ids::data_->length(n) + 1);
 		if (config.masking == 1)
 			Masking::get().bit_to_hard_mask(ref_seqs::data_->ptr(n), ref_seqs::data_->length(n), masked);
@@ -193,6 +207,8 @@ void Database_file::get_seq()
 	if (!all)
 		for (vector<string>::const_iterator i = config.seq_no.begin(); i != config.seq_no.end(); ++i)
 			seqs.insert(atoi(i->c_str()) - 1);
+	const size_t max_letters = config.chunk_size == 0.0 ? std::numeric_limits<size_t>::max() : (size_t)(config.chunk_size*1e9);
+	size_t letters = 0;
 	for (size_t n = 0; n < ref_header.sequences; ++n) {
 		read(&c, 1);
 		while (read(&c, 1), c != '\xff')
@@ -208,7 +224,22 @@ void Database_file::get_seq()
 			else
 				cout << sequence(seq) << endl;
 		}
+		letters += seq.size();
+		if (letters >= max_letters)
+			break;
 		seq.clear();
 		id.clear();
 	}
+}
+
+void db_info()
+{
+	Input_stream db_file(config.database);
+	Reference_header header;
+	Database_file::read_header(db_file, header);
+	cout << "Database format version = " << header.db_version << endl;
+	cout << "Diamond build = " << header.build << endl;
+	cout << "Sequences = " << header.sequences << endl;
+	cout << "Letters = " << header.letters << endl;
+	db_file.close();
 }
\ No newline at end of file
