@@ -55,9 +55,6 @@ struct Reference_header
 	uint64_t unique_id;
 	uint32_t build, db_version;
 	uint64_t sequences, letters, pos_array_offset;
-#ifdef EXTRA
-	Sequence_type sequence_type;
-#endif
 	enum { current_db_version = 1 };
 };
 
@@ -74,20 +71,22 @@ struct Database_file : public Input_stream
 	Database_file():
 		Input_stream (config.database)
 	{
-		if(this->read(&ref_header, 1) != 1)
-			throw Database_format_exception ();
-		if(ref_header.unique_id != Reference_header ().unique_id)
-			throw Database_format_exception ();
+		read_header(*this, ref_header);
 		if(ref_header.build < min_build_required || ref_header.db_version != Reference_header::current_db_version)
 			throw invalid_database_version_exception();
 		if (ref_header.sequences == 0)
 			throw std::runtime_error("Incomplete database file. Database building did not complete successfully.");
-#ifdef EXTRA
-		if(sequence_type(_val()) != ref_header.sequence_type)
-			throw std::runtime_error("Database has incorrect sequence type for current alignment mode.");
-#endif
 		pos_array_offset = ref_header.pos_array_offset;
 	}
+
+	static void read_header(Input_stream &stream, Reference_header &header)
+	{
+		if (stream.read(&header, 1) != 1)
+			throw Database_format_exception();
+		if (header.unique_id != Reference_header().unique_id)
+			throw Database_format_exception();
+	}
+
 	void rewind()
 	{
 		pos_array_offset = ref_header.pos_array_offset;
@@ -129,6 +128,8 @@ inline size_t max_id_len(const String_set<0> &ids)
 	return max;
 }
 
+string* get_allseqids(const char *s);
+
 struct Ref_map
 {
 	Ref_map():
@@ -157,10 +158,13 @@ struct Ref_map
 			n = next_++;
 			data_[block][i] = n;
 			len_.push_back((uint32_t)ref_seqs::get().length(i));
+			const char *title = ref_ids::get()[i].c_str();
 			if (config.salltitles)
-				name_.push_back(new string(ref_ids::get()[i].c_str()));
+				name_.push_back(new string(title));
+			else if (config.sallseqid)
+				name_.push_back(get_allseqids(title));
 			else
-				name_.push_back(get_str(ref_ids::get()[i].c_str(), Const::id_delimiters));
+				name_.push_back(get_str(title, Const::id_delimiters));
 			mtx_.unlock();
 		}
 		return n;
