@@ -204,7 +204,7 @@ private:
 			this->push_back(0);
 			for(unsigned i=0;i<Const::seedp;++i) {
 #ifdef EXTRA
-				log_stream << i << ' ' << partition_size(hst, i) << endl;
+				//log_stream << i << ' ' << partition_size(hst, i) << endl;
 #endif
 				this->push_back(this->operator[](i) + (range.contains(i) ? partition_size(hst, i) : 0));
 			}
