@@ -22,9 +22,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <utility>
 #include <map>
 #include <list>
+#include <stdint.h>
+#include <limits>
 #include "../basic/match.h"
-#include "../align/align.h"
 #include "score_profile.h"
+#include "../basic/translated_position.h"
+
+using std::list;
 
 void init_cbs();
 
@@ -41,7 +45,7 @@ struct Bias_correction : public vector<float>
 	{
 		score += (*this)[query_anchor + i*mult];
 	}
-	int operator()(const Hsp_data &hsp) const;
+	int operator()(const Hsp &hsp) const;
 	int operator()(const Diagonal_segment &d) const;
 };
 
@@ -65,6 +69,26 @@ struct Seed_hit
 	{
 		return ungapped.score > rhs.ungapped.score;
 	}
+	bool is_enveloped(list<Hsp>::const_iterator begin, list<Hsp>::const_iterator end, int dna_len) const
+	{
+		const DiagonalSegment d(ungapped, ::Frame(frame_));
+		for (list<Hsp>::const_iterator i = begin; i != end; ++i)
+			if (i->envelopes(d, dna_len))
+				return true;
+		return false;
+	}
+	DiagonalSegment diagonal_segment() const
+	{
+		return DiagonalSegment(ungapped, ::Frame(frame_));
+	}
+	interval query_source_range(int dna_len) const
+	{
+		return diagonal_segment().query_absolute_range(dna_len);
+	}
+	Strand strand() const
+	{
+		return ::Frame(frame_).strand;
+	}
 	static bool compare_pos(const Seed_hit &x, const Seed_hit &y)
 	{
 		return Diagonal_segment::cmp_subject_end(x.ungapped, y.ungapped);
@@ -73,6 +97,10 @@ struct Seed_hit
 	{
 		return x.frame_ < y.frame_ || (x.frame_ == y.frame_ && (x.diagonal() < y.diagonal() || (x.diagonal() == y.diagonal() && x.ungapped.j < y.ungapped.j)));
 	}
+	static bool compare_diag_strand(const Seed_hit &x, const Seed_hit &y)
+	{
+		return x.strand() < y.strand() || (x.strand() == y.strand() && (x.diagonal() < y.diagonal() || (x.diagonal() == y.diagonal() && x.ungapped.j < y.ungapped.j)));
+	}
 	struct Frame
 	{
 		unsigned operator()(const Seed_hit &x) const
@@ -94,6 +122,11 @@ struct Hsp_traits
 		score(0),
 		frame((int)frame)
 	{}
+	Hsp_traits(const interval &query_source_range):
+		query_source_range(query_source_range)
+	{}
+	Hsp_traits(const Hsp &hsp)
+	{}
 	int partial_score(const Diagonal_segment &d) const
 	{
 		const double overlap = std::max(d.subject_range().overlap_factor(subject_range), d.query_range().overlap_factor(query_range));
@@ -142,11 +175,11 @@ struct Hsp_traits
 		}
 	};
 	int d_min, d_max, score, frame;
-	interval query_range, subject_range;
+	interval query_source_range, query_range, subject_range;
 };
 
-template<typename _score>
-void smith_waterman(const Letter *query, local_match &segment, _score gap_open, _score gap_extend, vector<char> &transcript_buf, const _score& = int());
+/*template<typename _score>
+void smith_waterman(const Letter *query, local_match &segment, _score gap_open, _score gap_extend, vector<char> &transcript_buf, const _score& = int());*/
 int smith_waterman(const sequence &query, const sequence &subject, unsigned band, unsigned padding, int op, int ep);
 
 int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned seed_len, unsigned &delta, unsigned &len);
@@ -158,8 +191,8 @@ Diagonal_segment xdrop_ungapped(const sequence &query, const sequence &subject,
 struct Local {};
 struct Global {};
 
-int greedy_align(sequence query, const Long_score_profile &qp, const Bias_correction &query_bc, sequence subject, vector<Seed_hit>::const_iterator begin, vector<Seed_hit>::const_iterator end, bool log, std::list<Hsp_data> &hsps, std::list<Hsp_traits> &ts, unsigned frame);
-int greedy_align(sequence query, const Long_score_profile &qp, const Bias_correction &query_bc, sequence subject, bool log, std::list<Hsp_data> &hsps, std::list<Hsp_traits>::const_iterator t_begin, std::list<Hsp_traits>::const_iterator t_end, std::list<Hsp_traits> &ts, int cutoff, unsigned frame);
+int greedy_align(sequence query, const Long_score_profile &qp, const Bias_correction &query_bc, sequence subject, vector<Seed_hit>::const_iterator begin, vector<Seed_hit>::const_iterator end, bool log, std::list<Hsp> &hsps, std::list<Hsp_traits> &ts, unsigned frame);
+int greedy_align(sequence query, const Long_score_profile &qp, const Bias_correction &query_bc, sequence subject, bool log, std::list<Hsp> &hsps, std::list<Hsp_traits>::const_iterator t_begin, std::list<Hsp_traits>::const_iterator t_end, std::list<Hsp_traits> &ts, int cutoff, unsigned frame);
 int estimate_score(const Long_score_profile &qp, sequence s, int d, int d1, bool log);
 
 template<typename _t>
@@ -285,7 +318,7 @@ struct Diagonal_node : public Diagonal_segment
 struct Diag_graph
 {
 
-	enum { end = 0xffffffffffffffffllu };
+	enum { end = size_t(-1) };
 
 	struct Edge
 	{
@@ -441,6 +474,8 @@ private:
 	vector<uint8_t> data_;
 };
 
+extern size_t cells;
+
 struct Diag_scores {
 	enum {
 		block_len = 16
@@ -463,11 +498,66 @@ struct Diag_scores {
 	static int min_diag_score, min_low_score;
 };
 
-void smith_waterman(sequence q, sequence s, Hsp_data &out);
+struct DpTarget
+{
+	DpTarget()
+	{}
+	DpTarget(const sequence &seq):
+		seq(seq)
+	{}
+	DpTarget(const sequence &seq, int d_begin, int d_end, list<Hsp> *out = 0, int subject_id=0):
+		seq(seq),
+		d_begin(d_begin),
+		d_end(d_end),
+		subject_id(subject_id),
+		out(out)
+	{}
+	int left_i1() const
+	{
+		return std::max(d_end - 1, 0);
+	}
+	bool operator<(const DpTarget &x) const
+	{
+		return left_i1() < x.left_i1();
+	}
+	sequence seq;
+	int d_begin, d_end, score, subject_id;
+	bool overflow;
+	list<Hsp> *out;
+};
+
+struct DpStat
+{
+	DpStat():
+		gross_cells(0),
+		net_cells(0)
+	{}
+	DpStat& operator+=(DpStat &x)
+	{
+		mtx_.lock();
+		gross_cells += x.gross_cells;
+		net_cells += x.net_cells;
+		mtx_.unlock();
+		return *this;
+	}
+	size_t gross_cells, net_cells;
+private:
+	tthread::mutex mtx_;
+};
+
+extern DpStat dp_stat;
+
+void smith_waterman(sequence q, sequence s, Hsp &out);
 void smith_waterman(sequence q, sequence s, const Diag_graph &diags);
 int score_range(sequence query, sequence subject, int i, int j, int j_end);
 
-void swipe(const sequence &query, vector<sequence>::const_iterator subject_begin, vector<sequence>::const_iterator subject_end, vector<int>::iterator out);
-void banded_sw(const sequence &query, const sequence &subject, int d_begin, int d_end, int j_begin, int j_end, Hsp_data &out);
+void swipe(const sequence &query, vector<DpTarget>::const_iterator subject_begin, vector<DpTarget>::const_iterator subject_end, vector<int>::iterator out);
+void banded_sw(const sequence &query, const sequence &subject, int d_begin, int d_end, int j_begin, int j_end, Hsp &out);
 
-#endif /* FLOATING_SW_H_ */
+void anchored_3frame_dp(const TranslatedSequence &query, sequence &subject, const DiagonalSegment &anchor, Hsp &out, int gap_open, int gap_extend, int frame_shift);
+int sw_3frame(const TranslatedSequence &query, Strand strand, const sequence &subject, int gap_open, int gap_extend, int frame_shift, Hsp &out);
+
+void banded_swipe(const sequence &query, vector<DpTarget>::iterator target_begin, vector<DpTarget>::iterator target_end);
+void banded_3frame_swipe(const TranslatedSequence &query, Strand strand, vector<DpTarget>::iterator target_begin, vector<DpTarget>::iterator target_end, DpStat &stat, bool score_only);
+
+#endif /* FLOATING_SW_H_ */
\ No newline at end of file
