@@ -19,7 +19,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <vector>
 #include "dp.h"
 #include "../util/double_buffer.h"
-#include "growing_buffer.h"
 #include "../util/util.h"
 #include "traceback.h"
 #include "../output/output_format.h"
@@ -219,7 +218,7 @@ int needleman_wunsch(sequence query, sequence subject, int qbegin, int qend, int
 	return score;
 }
 
-void smith_waterman(sequence q, sequence s, Hsp_data &out)
+void smith_waterman(sequence q, sequence s, Hsp &out)
 {
 	int max_score;
 	const Fixed_score_buffer<int> &dp = needleman_wunsch(q, s, max_score, Local(), int());
@@ -292,20 +291,20 @@ void print_diag(int i0, int j0, int l, int score, const Diag_graph &diags, const
 
 void smith_waterman(sequence q, sequence s, const Diag_graph &diags)
 {
-	Hsp_data hsp;
+	Hsp hsp;
 	smith_waterman(q, s, hsp);
-	Hsp_data::Iterator i = hsp.begin();
+	Hsp::Iterator i = hsp.begin();
 	int i0 = -1, j0 = -1, l = 0, score = 0;
 	for (; i.good(); ++i) {
 		switch (i.op()) {
 		case op_match:
 		case op_substitution:
 			if (i0 < 0) {
-				i0 = i.query_pos;
+				i0 = i.query_pos.translated;
 				j0 = i.subject_pos;
 				l = 0;
 			}
-			score += score_matrix(q[i.query_pos], s[i.subject_pos]);
+			score += score_matrix(q[i.query_pos.translated], s[i.subject_pos]);
 			++l;
 			break;
 		case op_deletion:
@@ -318,8 +317,12 @@ void smith_waterman(sequence q, sequence s, const Diag_graph &diags)
 			}
 			else
 				score -= score_matrix.gap_extend();
+			break;
+		case op_frameshift_forward:
+		case op_frameshift_reverse:
+			;
 		}
 	}
 	print_diag(i0, j0, l, score, diags, q, s);
-	print_hsp(hsp, q);
+	print_hsp(hsp, TranslatedSequence(q));
 }
\ No newline at end of file
