@@ -18,9 +18,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "output_format.h"
 
-void Pairwise_format::print_match(const Hsp_context& r, Text_buffer &out) const
+void Pairwise_format::print_match(const Hsp_context& r, TextBuffer &out)
 {
 	static const unsigned width = 60;
+	const int dna_len = (int)r.query.source().length();
+	const Strand strand = r.frame() < 3 ? FORWARD : REVERSE;
 	out << '>';
 	Output_format::print_title(out, r.subject_name, true, true, " ");
 	out << "\nLength=" << r.subject_len << "\n\n";
@@ -32,17 +34,17 @@ void Pairwise_format::print_match(const Hsp_context& r, Text_buffer &out) const
 	if (align_mode.query_translated)
 		out << " Frame = " << r.blast_query_frame() << '\n';
 	out << '\n';
-	const unsigned digits = (unsigned)std::max(ceil(log10(r.subject_range().end_)), ceil(log10(r.query_range().end_)));
+	const unsigned digits = (unsigned)std::max(ceil(log10(r.subject_range().end_)), ceil(log10(r.query_source_range().end_)));
 
 	Hsp_context::Iterator qi = r.begin(), mi = r.begin(), si = r.begin();
 	while (qi.good()) {
 		out << "Query  ";
-		out.print(qi.query_pos+1, digits);
+		out.print(qi.query_pos.absolute(dna_len) + 1, digits);
 		out << "  ";
 		for (unsigned i = 0; i < width && qi.good(); ++i, ++qi)
 			out << qi.query_char();
 		out << " ";
-		out.print(qi.query_pos, 0);
+		out.print(TranslatedPosition::oriented_position(qi.query_pos.in_strand() - 1, strand, dna_len) + 1, 0);
 		out << '\n';
 
 		for (unsigned i = 0; i < digits + 9; ++i)
@@ -67,12 +69,11 @@ void Pairwise_format::print_footer(Output_stream &out) const
 
 }
 
-void Pairwise_format::print_query_epilog(Text_buffer &out, bool unaligned) const
+void Pairwise_format::print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned) const
 {
-
 }
 
-void Pairwise_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, Text_buffer &out, bool unaligned) const
+void Pairwise_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const
 {
 	out << "Query= " << query_name << "\n\nLength=" << query_len << "\n\n";
 	if (unaligned) {
