@@ -93,19 +93,19 @@ Blast_tab_format::Blast_tab_format() :
 	}
 }
 
-void print_staxids(Text_buffer &out, const char *id)
+void print_staxids(TextBuffer &out, const char *id)
 {
-	const vector<string> t(tokenize(id, "\1"));	
 	std::set<unsigned> taxons;
-	for (vector<string>::const_iterator i = t.begin(); i < t.end(); ++i)
-		taxons.insert(taxonomy.get(Taxonomy::Accession(*i)));
+	taxonomy.get_taxids(id, taxons);
+	if (taxons.empty())
+		return;
 	std::set<unsigned>::const_iterator i = taxons.begin();
 	out << *(i++);
 	for (; i != taxons.end(); ++i)
 		out << ';' << *i;
 }
 
-void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
+void Blast_tab_format::print_match(const Hsp_context& r, TextBuffer &out)
 {
 	for (vector<unsigned>::const_iterator i = fields.begin(); i != fields.end(); ++i) {
 		switch (*i) {
@@ -113,7 +113,7 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 			out.write_until(r.query_name, Const::id_delimiters);
 			break;
 		case 4:
-			out << r.source_query.length();
+			out << r.query.source().length();
 			break;
 		case 5:
 			print_title(out, r.subject_name, false, false, "<>");
@@ -137,7 +137,7 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 			out << r.subject_range().end_;
 			break;
 		case 17:
-			r.source_query.print(out, r.query_source_range().begin_, r.query_source_range().end_, input_value_traits);
+			r.query.source().print(out, r.query_source_range().begin_, r.query_source_range().end_, input_value_traits);
 			break;
 		case 18:
 		{
@@ -194,6 +194,8 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 					++n_matches;
 					break;
 				case op_substitution:
+				case op_frameshift_forward:
+				case op_frameshift_reverse:
 					if (n_matches > 0) {
 						out << n_matches;
 						n_matches = 0;
@@ -230,7 +232,7 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 			print_title(out, r.subject_name, true, true, "<>");
 			break;
 		case 43:
-			out << (double)r.query_source_range().length()*100.0 / r.source_query.length();
+			out << (double)r.query_source_range().length()*100.0 / r.query.source().length();
 			break;
 		case 45:
 			out << r.query_name;
@@ -250,7 +252,7 @@ void Blast_tab_format::print_match(const Hsp_context& r, Text_buffer &out) const
 	out << '\n';
 }
 
-void Blast_tab_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, Text_buffer &out, bool unaligned) const
+void Blast_tab_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const
 {
 	if (unaligned && config.report_unaligned == 1) {
 		for (vector<unsigned>::const_iterator i = fields.begin(); i != fields.end(); ++i) {
