@@ -19,9 +19,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "daa_record.h"
 #include "output.h"
 
-Binary_buffer::Iterator DAA_query_record::init(const Binary_buffer &buf)
+BinaryBuffer::Iterator DAA_query_record::init(const BinaryBuffer &buf)
 {
-	Binary_buffer::Iterator it(buf.begin());
+	BinaryBuffer::Iterator it(buf.begin());
 	uint32_t query_len;
 	it >> query_len;
 	it >> query_name;
@@ -30,17 +30,19 @@ Binary_buffer::Iterator DAA_query_record::init(const Binary_buffer &buf)
 	if (file_.mode() == Align_mode::blastp) {
 		Packed_sequence seq(it, query_len, false, 5);
 		seq.unpack(context[0], 5, query_len);
+		query_seq = TranslatedSequence(sequence(context[0]));
 	}
 	else {
 		const bool have_n = (flags & 1) == 1;
 		Packed_sequence seq(it, query_len, have_n, have_n ? 3 : 2);
 		seq.unpack(source_seq, have_n ? 3 : 2, query_len);
 		translate_query(source_seq, context);
+		query_seq = TranslatedSequence(sequence(source_seq), context);
 	}
 	return it;
 }
 
-Binary_buffer::Iterator& operator>>(Binary_buffer::Iterator &it, DAA_query_record::Match &r)
+BinaryBuffer::Iterator& operator>>(BinaryBuffer::Iterator &it, DAA_query_record::Match &r)
 {
 	const uint32_t old_subject = r.subject_id;
 	it >> r.subject_id;
@@ -68,11 +70,11 @@ Binary_buffer::Iterator& operator>>(Binary_buffer::Iterator &it, DAA_query_recor
 		r.frame = 0;
 		r.query_range.begin_ = query_begin;
 	}
-	r.context().parse().set_query_source_range(query_begin);
+	r.context().parse();
 	return it;
 }
 
-Hsp_data::Hsp_data(const Intermediate_record &r, unsigned query_source_len):
+Hsp::Hsp(const IntermediateRecord &r, unsigned query_source_len):
 	score(r.score),
 	transcript(r.transcript)
 {
