@@ -25,8 +25,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../basic/value.h"
 #include "../basic/translate.h"
 #include "../basic/packed_transcript.h"
-#include "../align/match_func.h"
 #include "../basic/score_matrix.h"
+#include "../basic/match.h"
 
 using std::string;
 using std::vector;
@@ -45,7 +45,7 @@ inline void translate_query(const vector<Letter>& query, vector<Letter> *context
 struct DAA_query_record
 {
 
-	struct Match : public Hsp_data
+	struct Match : public Hsp
 	{
 
 		Match(const DAA_query_record &query_record) :
@@ -58,8 +58,7 @@ struct DAA_query_record
 		{
 			return Hsp_context(*this,
 				(unsigned)parent_.query_num,
-				sequence(parent_.context[frame]),
-				align_mode.query_translated ? sequence(parent_.source_seq) : sequence(parent_.context[0]),
+				parent_.query_seq,
 				parent_.query_name.c_str(),
 				subject_id,
 				subject_id,
@@ -75,13 +74,13 @@ struct DAA_query_record
 	private:
 
 		const DAA_query_record &parent_;
-		friend Binary_buffer::Iterator& operator>>(Binary_buffer::Iterator &it, Match &r);
+		friend BinaryBuffer::Iterator& operator>>(BinaryBuffer::Iterator &it, Match &r);
 
 	};
 
 	struct Match_iterator
 	{
-		Match_iterator(const DAA_query_record &parent, Binary_buffer::Iterator it) :
+		Match_iterator(const DAA_query_record &parent, BinaryBuffer::Iterator it) :
 			r_(parent),
 			it_(it),
 			good_(true)
@@ -106,11 +105,11 @@ struct DAA_query_record
 		}
 	private:
 		Match r_;
-		Binary_buffer::Iterator it_;
+		BinaryBuffer::Iterator it_;
 		bool good_;
 	};
 
-	DAA_query_record(const DAA_file& file, const Binary_buffer &buf, size_t query_num):
+	DAA_query_record(const DAA_file& file, const BinaryBuffer &buf, size_t query_num):
 		query_num (query_num),
 		file_(file),
 		it_(init(buf))
@@ -130,18 +129,19 @@ struct DAA_query_record
 	size_t query_num;
 	vector<Letter> source_seq;
 	vector<Letter> context[6];
+	TranslatedSequence query_seq;
 
 private:
 
-	Binary_buffer::Iterator init(const Binary_buffer &buf);
+	BinaryBuffer::Iterator init(const BinaryBuffer &buf);
 
 	const DAA_file& file_;
-	const Binary_buffer::Iterator it_;
+	const BinaryBuffer::Iterator it_;
 	
-	friend Binary_buffer::Iterator& operator>>(Binary_buffer::Iterator &it, Match &r);
+	friend BinaryBuffer::Iterator& operator>>(BinaryBuffer::Iterator &it, Match &r);
 
 };
 
-Binary_buffer::Iterator& operator>>(Binary_buffer::Iterator &it, DAA_query_record::Match &r);
+BinaryBuffer::Iterator& operator>>(BinaryBuffer::Iterator &it, DAA_query_record::Match &r);
 
 #endif /* DAA_RECORD_H_ */
