@@ -20,47 +20,68 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../data/taxonomy.h"
 #include "output_format.h"
 #include "../data/reference.h"
+#include "../util/escape_sequences.h"
 
 using std::endl;
 
 auto_ptr<Output_format> output_format;
 
-void Output_format::print_title(Text_buffer &buf, const char *id, bool full_titles, bool all_titles, const char *separator)
+void print_escaped_until(TextBuffer &buf, const char *s, const char *delimiters, const EscapeSequences *esc)
+{
+	if (esc == 0)
+		buf.write_until(s, delimiters);
+	else {
+		string tmp;
+		esc->escape(s, find_first_of(s, delimiters), tmp);
+		buf << tmp;
+	}
+}
+
+void print_escaped(TextBuffer &buf, const string &s, const EscapeSequences *esc)
+{
+	if (esc == 0)
+		buf << s;
+	else {
+		string tmp;
+		esc->escape(s, tmp);
+		buf << tmp;
+	}
+}
+
+void Output_format::print_title(TextBuffer &buf, const char *id, bool full_titles, bool all_titles, const char *separator, const EscapeSequences *esc)
 {
 	if (!all_titles) {
-		buf.write_until(id, full_titles ? "\1" : Const::id_delimiters);
+		print_escaped_until(buf, id, full_titles ? "\1" : Const::id_delimiters, esc);
 		return;
 	}
 	if (strchr(id, '\1') == 0) {
-		buf.write_until(id, full_titles ? "\1" : Const::id_delimiters);
+		print_escaped_until(buf, id, full_titles ? "\1" : Const::id_delimiters, esc);
 		return;
 	}
-	//size_t n = 0;
 	const vector<string> t(tokenize(id, "\1"));
 	vector<string>::const_iterator i = t.begin();
-	for (; i<t.end() - 1; ++i) {
-		if (full_titles)
-			buf << *i << separator;
+	for (; i < t.end() - 1; ++i) {
+		if (full_titles) {
+			print_escaped(buf, *i, esc);
+			buf << separator;
+		}
 		else {
-			buf.write_until(i->c_str(), Const::id_delimiters);
+			print_escaped_until(buf, i->c_str(), Const::id_delimiters, esc);
 			buf << ";";
 		}
-		//n += i->length() + 2;
 	}
 	if (full_titles)
-		buf << *i;
+		print_escaped(buf, *i, esc);
 	else
-		buf.write_until(i->c_str(), Const::id_delimiters);
-	//n += i->length();
-	//return n;
+		print_escaped_until(buf, i->c_str(), Const::id_delimiters, esc);
 }
 
-void print_hsp(Hsp_data &hsp, sequence query)
+void print_hsp(Hsp &hsp, const TranslatedSequence &query)
 {
-	Text_buffer buf;
-	Pairwise_format().print_match(Hsp_context(hsp, 0, query, query, "", 0, 0, "", 0, 0, 0), buf);
+	TextBuffer buf;
+	Pairwise_format().print_match(Hsp_context(hsp, 0, query, "", 0, 0, "", 0, 0, 0), buf);
 	buf << '\0';
-	cout << buf.get_begin();
+	cout << buf.get_begin() << endl;
 }
 
 Output_format* get_output_format()
@@ -84,23 +105,57 @@ Output_format* get_output_format()
 		return new Pairwise_format;
 	else if (f[0] == "null")
 		return new Null_format;
+	else if (f[0] == "102")
+		return new Taxon_format;
 	else
-		throw std::runtime_error("Invalid output format. Allowed values: 5,6,100,101");
+		throw std::runtime_error("Invalid output format. Allowed values: 0,5,6,100,101,102");
 }
 
+void init_output()
+{
+	output_format = auto_ptr<Output_format>(get_output_format());
+	if (*output_format == Output_format::taxon && config.toppercent == 100.0)
+		config.toppercent = 10.0;
+	if (config.toppercent == 100.0) {
+		message_stream << "#Target sequences to report alignments for: ";
+		if (config.max_alignments == 0) {
+			config.max_alignments = std::numeric_limits<uint64_t>::max();
+			message_stream << "unlimited" << endl;
+		}
+		else
+			message_stream << config.max_alignments << endl;
+	}
+	else
+		message_stream << "Percentage range of top alignment score to report hits: " << config.toppercent << endl;
+}
 
-void XML_format::print_match(const Hsp_context &r, Text_buffer &out) const
+void XML_format::print_match(const Hsp_context &r, TextBuffer &out)
 {
 	if(r.hsp_num == 0) {
 		if (r.hit_num > 0)
 			out << "  </Hit_hsps>" << '\n' << "</Hit>" << '\n';
+
+		out << "<Hit>" << '\n'
+			<< "  <Hit_num>" << r.hit_num + 1 << "</Hit_num>" << '\n';
 		string id, def;
 		get_title_def(r.subject_name, id, def);
-		out << "<Hit>" << '\n'
-			<< "  <Hit_num>" << r.hit_num + 1 << "</Hit_num>" << '\n'
-			<< "  <Hit_id>" << id << "</Hit_id>" << '\n'
-			<< "  <Hit_def>" << def << "</Hit_def>" << '\n';
-		out << "  <Hit_accession>" << get_accession(id) << "</Hit_accession>" << '\n'
+		if (config.xml_blord_format) {
+			out << "  <Hit_id>gnl|BL_ORD_ID|" << r.subject_id << "</Hit_id>" << '\n'
+				<< "  <Hit_def>";
+			Output_format::print_title(out, r.subject_name, true, true, " &gt;", &EscapeSequences::XML);
+			out << "</Hit_def>" << '\n';
+		}
+		else {
+			out << "  <Hit_id>";
+			print_escaped(out, id, &EscapeSequences::XML);
+			out << "</Hit_id>" << '\n'
+				<< "  <Hit_def>";
+			Output_format::print_title(out, def.c_str(), true, true, " &gt;", &EscapeSequences::XML);
+			out << "</Hit_def>" << '\n';
+		}
+		out << "  <Hit_accession>";
+		print_escaped(out, get_accession(id), &EscapeSequences::XML);
+		out << "</Hit_accession>" << '\n'
 			<< "  <Hit_len>" << r.subject_len << "</Hit_len>" << '\n'
 			<< "  <Hit_hsps>" << '\n';
 	}
@@ -152,11 +207,13 @@ void XML_format::print_header(Output_stream &f, int mode, const char *matrix, in
 		<< "  <BlastOutput_program>" << mode_str(mode) << "</BlastOutput_program>" << endl
 		<< "  <BlastOutput_version>" << Const::program_name << ' ' << Const::version_string << "</BlastOutput_version>" << endl
 		<< "  <BlastOutput_reference>Benjamin Buchfink, Xie Chao, and Daniel Huson (2015), &quot;Fast and sensitive protein alignment using DIAMOND&quot;, Nature Methods 12:59-60.</BlastOutput_reference>" << endl
-		<< "  <BlastOutput_db></BlastOutput_db>" << endl
+		<< "  <BlastOutput_db>" << config.database << "</BlastOutput_db>" << endl
 		<< "  <BlastOutput_query-ID>Query_1</BlastOutput_query-ID>" << endl
 		<< "  <BlastOutput_query-def>";
 	const string fqn = string(first_query_name);
-	ss << fqn.substr(0, fqn.find('\1'));
+	string escaped;
+	EscapeSequences::XML.escape(fqn, escaped);
+	ss << escaped.substr(0, fqn.find('\1'));
 		ss << "</BlastOutput_query-def>" << endl << "  <BlastOutput_query-len>" << first_query_len << "</BlastOutput_query-len>" << endl
 		<< "  <BlastOutput_param>" << endl
 		<< "    <Parameters>" << endl
@@ -164,26 +221,26 @@ void XML_format::print_header(Output_stream &f, int mode, const char *matrix, in
 		<< "      <Parameters_expect>" << evalue << "</Parameters_expect>" << endl
 		<< "      <Parameters_gap-open>" << gap_open << "</Parameters_gap-open>" << endl
 		<< "      <Parameters_gap-extend>" << gap_extend << "</Parameters_gap-extend>" << endl
-		<< "      <Parameters_filter></Parameters_filter>" << endl
+		<< "      <Parameters_filter>F</Parameters_filter>" << endl
 		<< "    </Parameters>" << endl
 		<< "  </BlastOutput_param>" << endl
 		<< "<BlastOutput_iterations>" << endl;
 	f.write(ss.str().c_str(), ss.str().length());
 }
 
-void XML_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, Text_buffer &out, bool unaligned) const
+void XML_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const
 {
 	out << "<Iteration>" << '\n'
 		<< "  <Iteration_iter-num>" << query_num + 1 << "</Iteration_iter-num>" << '\n'
 		<< "  <Iteration_query-ID>Query_" << query_num + 1 << "</Iteration_query-ID>" << '\n'
 		<< "  <Iteration_query-def>";
-	print_title(out, query_name, true, false, "");
+	print_title(out, query_name, true, false, "", &EscapeSequences::XML);
 	out << "</Iteration_query-def>" << '\n'
 		<< "  <Iteration_query-len>" << query_len << "</Iteration_query-len>" << '\n'
 		<< "<Iteration_hits>" << '\n';
 }
 
-void XML_format::print_query_epilog(Text_buffer &out, bool unaligned) const
+void XML_format::print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned) const
 {
 	if (!unaligned) {
 		out << "  </Hit_hsps>" << '\n'
