@@ -22,7 +22,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../data/queries.h"
 #include "../basic/statistics.h"
 #include "../basic/shape_config.h"
-#include "../align/align_queries.h"
 #include "../search/align_range.h"
 #include "../util/seq_file_format.h"
 #include "../data/load_seqs.h"
@@ -165,7 +164,7 @@ void run_ref_chunk(Database_file &db_file,
 	delete Trace_pt_buffer::instance;
 
 	if (blocked_processing) {
-		Intermediate_record::finish_file(*out);
+		IntermediateRecord::finish_file(*out);
 		delete out;
 	}
 
@@ -320,7 +319,7 @@ void master_thread_di()
 	timer2.start();
 
 	align_mode = Align_mode(Align_mode::from_command(config.command));
-	output_format = auto_ptr<Output_format>(get_output_format());
+	init_output();
 
 	message_stream << "Temporary directory: " << Temp_file::get_temp_dir() << endl;
 
@@ -343,12 +342,7 @@ void master_thread_di()
 	Config::set_option(config.db_size, (uint64_t)ref_header.letters);
 
 	set_max_open_files(config.query_bins * config.threads_ + unsigned(ref_header.letters / (size_t)(config.chunk_size * 1e9)) + 16);
-
-	if (!config.prot_accession2taxid.empty()) {
-		timer.go("Loading taxonomy");
-		taxonomy.load();
-		timer.finish();
-	}
+	taxonomy.init();
 
 	master_thread(db_file, timer2);
 }
\ No newline at end of file
