@@ -18,7 +18,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <iostream>
 #include "../basic/config.h"
-#include "../output/view.h"
 #include "tools.h"
 #include "../extra/compare.h"
 
@@ -31,10 +30,14 @@ void model_seqs();
 void opt();
 void run_masker();
 void fastq2fasta();
+void view();
+void make_db();
+void db_info();
+void test_main();
+void benchmark_sw();
 
 int main(int ac, const char* av[])
 {
-
 	try {
 
 		check_simd();
@@ -59,9 +62,6 @@ int main(int ac, const char* av[])
 		case Config::getseq:
 			get_seq();
 			break;
-		case Config::benchmark:
-			benchmark_sw();
-			break;
 		case Config::random_seqs:
 			random_seqs();
 			break;
@@ -95,6 +95,17 @@ int main(int ac, const char* av[])
 		case Config::fastq2fasta:
 			fastq2fasta();
 			break;
+		case Config::dbinfo:
+			db_info();
+			break;
+#ifdef EXTRA
+		case Config::test_extra:
+			test_main();
+			break;
+		case Config::benchmark:
+			benchmark_sw();
+			break;
+#endif
 		default:
 			return 1;
 		}
