@@ -1,25 +1,80 @@
 /****
-Copyright (c) 2016, Benjamin Buchfink
-All rights reserved.
+DIAMOND protein aligner
+Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #include "align_range.h"
 #include "../data/reference.h"
 #include "../basic/config.h"
 
+void setup_search_cont()
+{
+	unsigned index_mode;
+	if (config.mode_more_sensitive) {
+		index_mode = 11;
+	}
+	else if (config.mode_sensitive) {
+		index_mode = 11;
+	}
+	else {
+		index_mode = 10;
+		Reduction::reduction = Reduction("KR EQ D N C G H F Y IV LM W P S T A");
+	}
+	::shapes = shape_config(index_mode, 1, vector<string>());
+}
+
+void setup_search()
+{
+	if (config.algo == Config::double_indexed) {
+		if (config.mode_more_sensitive) {
+			Config::set_option(config.index_mode, 9u);
+			Config::set_option(config.freq_sd, 200.0);
+		}
+		else if (config.mode_sensitive) {
+			Config::set_option(config.index_mode, 9u);
+			Config::set_option(config.freq_sd, 10.0);
+		}
+		else {
+			Config::set_option(config.index_mode, 8u);
+			Config::set_option(config.freq_sd, 50.0);
+		}
+		Reduction::reduction = Reduction("A KR EDNQ C G H ILVM FYW P ST");
+		::shapes = shape_config(config.index_mode, config.shapes, config.shape_mask);
+	}
+	else {
+		if (config.mode_more_sensitive) {
+			Config::set_option(config.freq_sd, 200.0);
+		}
+		else if (config.mode_sensitive) {
+			Config::set_option(config.freq_sd, 20.0);
+		}
+		else {
+			Config::set_option(config.freq_sd, 50.0);
+		}
+		config.lowmem = 1;
+	}
+
+	message_stream << "Algorithm: " << (config.algo == Config::double_indexed ? "Double-indexed" : "Query-indexed") << endl;
+	verbose_stream << "Reduction: " << Reduction::reduction << endl;
+
+	verbose_stream << "Seed frequency SD: " << config.freq_sd << endl;
+	verbose_stream << "Shape configuration: " << ::shapes << endl;
+	config.seed_anchor = std::min(::shapes[0].length_ - 1, 8u);
+}
+
 void setup_search_params(pair<size_t, size_t> query_len_bounds, size_t chunk_db_letters)
 {
 	const double b = config.min_bit_score == 0 ? score_matrix.bitscore(config.max_evalue, ref_header.letters, (unsigned)query_len_bounds.first) : config.min_bit_score;
