@@ -61,52 +61,6 @@ struct partition
 	{ return i < remainder ? (size + 1) : size; }
 };
 
-struct interval
-{
-	interval():
-		begin_ (0),
-		end_ (0)
-	{ }
-	interval(int begin, int end):
-		begin_ (begin),
-		end_ (end)
-	{ }
-	int length() const
-	{ return end_ > begin_ ? end_ - begin_ : 0; }
-	unsigned overlap(const interval &rhs) const
-	{ return intersect(*this, rhs).length(); }
-	double overlap_factor(const interval &rhs) const
-	{
-		return (double)overlap(rhs) / (double)length();
-	}
-	bool includes(int p) const
-	{ return p >= begin_ && p < end_; }
-	friend std::ostream& operator<<(std::ostream &os, const interval &x)
-	{ os << "[" << x.begin_ << ";" << x.end_ << "]"; return os; }
-	bool operator<(const interval &rhs) const
-	{
-		return begin_ < rhs.begin_;
-	}
-	friend interval intersect(const interval &lhs, const interval &rhs);
-	int begin_, end_;
-};
-
-inline interval intersect(const interval &lhs, const interval &rhs)
-{
-	return interval(std::max(lhs.begin_, rhs.begin_), std::min(lhs.end_, rhs.end_));
-}
-
-struct Interval_set : public vector<interval>
-{
-	int max_intersect(const interval &x) const
-	{
-		int max = 0;
-		for (const_iterator i = begin(); i < end(); ++i)
-			max = std::max(max, intersect(x, *i).length());
-		return max;
-	}
-};
-
 #ifdef __SSE2__
 inline void print(const __m128i &x)
 {
@@ -146,66 +100,6 @@ template<typename _t>
 inline _t round_up(_t x, _t m)
 { return div_up(x, m) * m; }
 
-template<typename _val, typename _dir>
-inline void print_seq(const _val* s, const _dir& d)
-{
-	unsigned i=0;
-	while(get_dir(s,i,d) != 0xff) {
-		std::cout << mask_critical(get_dir(s,i,d));
-		++i;
-	}
-}
-
-#ifdef _MSC_VER
-#define TLS_PTR __declspec(thread)
-#else
-#define TLS_PTR __thread
-#endif
-
-struct Ptr_wrapper_base
-{
-	virtual ~Ptr_wrapper_base()
-	{}
-};
-
-template<typename _t>
-struct Ptr_wrapper : public Ptr_wrapper_base
-{
-	Ptr_wrapper(_t *ptr) :
-		ptr(ptr)
-	{}
-	virtual ~Ptr_wrapper()
-	{
-		delete ptr;
-	}
-	_t *ptr;
-};
-
-struct TLS
-{
-	template<typename _t>
-	static _t& get(_t *&ptr)
-	{
-		if (ptr == 0) {
-			ptr = new _t;
-			if (ptr_ == 0)
-				ptr_ = new vector<Ptr_wrapper_base*>;
-			ptr_->push_back(new Ptr_wrapper<_t>(ptr));
-		}
-		return *ptr;
-	}
-	static void clear()
-	{
-		if (ptr_ == 0)
-			return;
-		for (vector<Ptr_wrapper_base*>::iterator i = ptr_->begin(); i != ptr_->end(); ++i)
-			delete *i;
-		delete ptr_;
-	}
-private:
-	static TLS_PTR vector<Ptr_wrapper_base*> *ptr_;
-};
-
 inline vector<string> tokenize(const char *str, const char *delimiters)
 {
 	vector<string> out;
@@ -388,18 +282,6 @@ private:
 	int n;
 };
 
-inline int ctz(uint64_t x)
-{
-#ifdef _MSC_VER
-	if (x)
-		return (int)__popcnt64((x ^ (x - 1)) >> 1);
-	else
-		return CHAR_BIT * sizeof(x);
-#else
-	return __builtin_ctzll(x);
-#endif
-}
-
 template<typename _t>
 inline void assign_ptr(_t& dst, _t *src)
 {
@@ -467,7 +349,7 @@ struct Matrix
 	}
 	_t* operator[](int i)
 	{
-		return data_[i*cols_];
+		return &data_[i*cols_];
 	}
 private:
 	int cols_;
